/* Copyright 2009 Steven Reynolds
 *  Demonstrate GUI, reactions */

import swing._
import event._

class Route(n:String,dist:double) {
    val name:String = n
    val distance:double = dist
    override def toString = name
}

object GUIEx  extends SimpleGUIApplication {
    val I10 = new Route("I-10",30)
    val I59 =  new Route("I-59", 25)
    val WestPark = new Route("Westpark Tollway",18)
    val result:Label = new Label("")
    val routeCombo = new ComboBox[Route](List(WestPark, I10, I59 ))
    var currentRoute = routeCombo.selection.item.asInstanceOf[Route]
    val departCombo = new ComboBox[Integer](List(5,6,7,8,9,10))
    var currentDepart = departCombo.selection.item.asInstanceOf[Integer]
    val inputPanel = new FlowPanel {
        contents.append(new Label("Route"), routeCombo,
                        new Label("Depart"), departCombo , new Label("AM"))
        border = Swing.EmptyBorder(15, 10, 10, 10)
    }
    val outputPanel = new FlowPanel {
        contents.append(new Label("Estimate:"),result )
        border = Swing.EmptyBorder(15, 10, 10, 10)
    }
    val ui = new BoxPanel(Orientation.Vertical) {
        contents.append(inputPanel,outputPanel)
        result.text = getDuration(currentRoute,currentDepart.doubleValue)
    }
    def getDuration(route:Route, depart:double):String = {
        val velocity = getVelocity(route,depart)
        if (velocity < 0.1) return "forever"
        (route.distance/velocity).toString + " hours"
    }
    def getVelocity(route:Route, depart:double):double = {
        if (route == I10 && depart < 10) return 0
        if (depart > 7.9 && depart < 8.5) return 20
        if (depart > 6.9 && depart < 9.1) return 40
        60;   
    }

    def updateEstimate() {
        currentDepart = departCombo.selection.item.asInstanceOf[Integer]
        result.text = getDuration(currentRoute,currentDepart.doubleValue)
    }

    def top = new MainFrame {
        title = "Route Selector"
        contents = ui
        reactions += {
            case SelectionChanged(`routeCombo`)  =>
                // The Scala way with closures
                currentRoute = routeCombo.selection.item.asInstanceOf[Route]
                result.text = getDuration(currentRoute,currentDepart.doubleValue)
            case SelectionChanged(`departCombo`)  =>
                // The Java way *without* closures
                updateEstimate()
        }
        listenTo(routeCombo.selection,departCombo.selection)
    }
}
